//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Ovladanie krokoveho motora
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

// Definicia cievok krokoveho motora
#define   C1    1           // prva cievka - PA0
#define   C2    1 << 1      // druha cievka - PA1  
#define   C3    1 << 2      // tretia cievka - PA2
#define   C4    1 << 3      // stvrta cievka - PA3

// Pocet krokov casovaca na 1 us
#define STEPS   ((unsigned int) ((MCK/8) + 500000) / 1000000L)

//  *****************************************************
//  Mikrosekundove oneskorenie (max. cca 10 000)
//  *****************************************************
void delay(unsigned int us)
{
  AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_TC0);  // povolenie hodin pre TC0
  AT91C_BASE_TC0->TC_CMR = 1;                      // volba hodin (MCK/8)
  AT91C_BASE_TC0->TC_RC = us * STEPS;              // nastavenie porovnavacieho registra C
  AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN |        // povolenie hodin
                           AT91C_TC_SWTRG;         // restart casovaca

  // cakanie na ukoncenie oneskorenia
  // ked je nastaveny compare C flag, je hotovo
  while ((AT91C_BASE_TC0->TC_SR & AT91C_TC_CPCS) == 0) asm ("nop");
    
  AT91C_BASE_PMC->PMC_PCDR = (1 << AT91C_ID_TC0);  // zakazanie hodin pre TC0
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{ int krok, ii;

  //******************* Uvodna inicializacia *******************

  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vstupne piny
  AT91C_BASE_PIOA->PIO_ODR = S2;
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = C1 | C2 | C3 | C4;
  // povolenie Pull-Up rezistoru
  AT91C_BASE_PIOA->PIO_PPUER = S2;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = S2 | C1 | C2 | C3 | C4;

  //******************* Hlavna cast programu *******************
  // cievky su vypnute
  AT91C_BASE_PIOA->PIO_CODR = C1 | C2 | C3 | C4;

  // zapnutie prvej cievky
  AT91C_BASE_PIOA->PIO_SODR = C1;
  krok = 0;
  // nekonecna slucka
  while(1)
  { // cakanie na stlacenie S2  
    while(AT91C_BASE_PIOA->PIO_PDSR & S2)     // kontrola, ci je S2 v log.1
      delay(200);                             // cakanie kvoli potlaceniu zakmitov
    // cakanie na pustenie S2
    while(!(AT91C_BASE_PIOA->PIO_PDSR & S2))  // kontrola, ci je S2 v log.0
      delay(200);                             // cakanie kvoli potlaceniu zakmitov

    // pootocenie o dany pocet krokov
    // napr. 90 stupnov je 50 krokov (1,8 stupna na krok)
    for(ii=0;ii<50;ii++)
    { // cakanie treba prisposobit schopnostiam motora
      delay(3000);

      krok++;
      switch(krok)
      { case 1: AT91C_BASE_PIOA->PIO_SODR = C2; // zapnutie dalsej cievky
                AT91C_BASE_PIOA->PIO_CODR = C1; // vypnutie predoslej cievky
                break;
        case 2: AT91C_BASE_PIOA->PIO_SODR = C3; // zapnutie dalsej cievky
                AT91C_BASE_PIOA->PIO_CODR = C2; // vypnutie predoslej cievky
                break;
        case 3: AT91C_BASE_PIOA->PIO_SODR = C4; // zapnutie dalsej cievky
                AT91C_BASE_PIOA->PIO_CODR = C3; // vypnutie predoslej cievky
                break;
        case 4: AT91C_BASE_PIOA->PIO_SODR = C1; // zapnutie dalsej cievky
                AT91C_BASE_PIOA->PIO_CODR = C4; // vypnutie predoslej cievky
                krok = 0;
                break;
      }
    }
  }
}
